@@ -16,7 +16,7 @@
 #endif
 
 #ifndef BUILD_GLES3
-/* #undef BUILD_GLES3 */
+#define BUILD_GLES3
 #endif
 
 // Miscellaneous flags
@@ -50,7 +50,7 @@
 // ENABLE flags
 
 #ifndef ENABLE_SCRIPTING
-#define ENABLE_SCRIPTING
+/* #undef ENABLE_SCRIPTING */
 #endif
 
 // USE flags
@@ -119,4 +119,10 @@
 #define USE_ZLIB
 #endif
 
+// HAVE flags
+
+#ifndef HAVE_POPCOUNT32
+/* #undef HAVE_POPCOUNT32 */
+#endif
+
 #endif
